import os.path
from   hashlib import md5
from   PyQt4   import QtCore

class DuplicateFinder(QtCore.QObject):
    
 
    def __init__(self, path):
        """\brief Constructor of the DuplicateFinder."""
        QtCore.QObject.__init__(self)

        self.path       = path
        self.data       = {}
        self.count      = 0
        self.dupCount   = 0


    def run(self):
        """\brief This method starts a DuplicateFinder."""

        # retrieve the number of files within the directory
        fileCount = 0.0
        for root, dirs, files in os.walk(self.path):
            fileCount += len(files)
    
        # check each file in the directory
        for root, dirs, files in os.walk(self.path):
            for file in files:
                self.checkFile(os.path.join(root, file))
                self.emit( QtCore.SIGNAL("progress"), 
                           100.0 * self.count / fileCount)
        
        
    def checkFile(self, filename):
        """\brief This method checks whether a file is a duplicate or not."""
#        print "check file %s" % filename
        
        self.count += 1
        
        md5 = self.createMD5(filename)
        
        # fehler
        if md5 == -1:
            return
                
        if self.data.has_key(md5):
            self.data[md5].append(filename)
            self.dupCount += 1
        else:
            self.data[md5] = [filename]
            
                
    def createMD5(self, filename, buf_size = 8096):
        """\brief This method calculates the md5 hash sum of a given file."""
        
        # ignore none existing files
        if not os.path.exists(filename):
            return -1

        # ignore empty files
        if os.stat(filename).st_size == 0:
            return -1
        
        # ignore files that can not be opened
        try:
            f = open(filename)
        except:
            return -1
        
        m = md5()

        # the with statement makes sure the file will be closed
        with f:
            
            # we read the file in small chunks until EOF
            data = f.read(buf_size)
            while data:
                
                # we add data to the md5 hash 
                m.update(data)
                data = f.read(buf_size)
        
        # we return the md5 hash in hexadecimal format
        return m.hexdigest()
    
    
    def getDuplicates(self):
        """\brief Returns the duplicates in a separate dictionary."""
        result = {}
        for key in self.data:
            filenames = self.data[key]            
            if len(filenames) > 1 and self.checkDuplicates(filenames):
                result[key] = filenames
        return result
            

    def checkDuplicates(self, files):
        """\brief This method checks whether a list of files is really 
        identical.
        """
        return True
    
    
    def cmpFileContent(self, file1, file2, buf_size = 8096):
        """\brief Compares two files whether their content is identical or not.
        """
        same = False
        fp1  = open(file1, 'rb')
        fp2  = open(file2, 'rb')
        
        # compare buffers as long as something to read is there
        while True:
            b1 = fp1.read(buf_size)
            b2 = fp2.read(buf_size)
            
            # break as soon as something is different
            if b1 != b2:
                break
            
            # EOF
            if not b1:
                same = True
                break
            
        fp1.close()
        fp2.close()
        return same 
    