from PyQt4              import QtCore
from PyQt4.QtGui        import QAction, QFileDialog, QMainWindow, QTableWidgetItem
from Ui_MainWindow      import Ui_MainWindow
from SearchThread       import SearchThread

class DuplicateFinderUI(QMainWindow):
    """\brief The DuplicateFinderUI class provides the main window for the 
              duplicate finder program.
    """
    
    
    def __init__(self):
        """\brief Constructor of the UI."""

        # constructor call of the parent class        
        QMainWindow.__init__(self)

        # setup ui
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.setupConnections()

        # set initial values
        self.ui.progressBar.setValue(0)
        self.ui.lineEdit.setText("C:\\Users\\ik3\\Documents")
        

    def setupConnections(self):
        """\brief This method creates the connections between the default ui 
                  elements and the methods of the main window.
        """
        
        # exit action
        self.exit = QAction('Beenden', self)
        self.exit.setShortcut('Ctrl+Q')
        self.connect( self.exit, QtCore.SIGNAL('triggered()'), 
                                 QtCore.SLOT('close()') )
        self.ui.menuFile.addAction(self.exit)
        
        # run button
        self.connect( self.ui.pushButton, QtCore.SIGNAL('clicked()'), 
                                          self.runDuplicateFinder )
        self.connect( self.ui.pushButton_2, QtCore.SIGNAL('clicked()'),
                                          self.setDirectory)


    def setDirectory(self):
        """\brief This method sets the directory with the help of a dialog."""
        self.ui.lineEdit.setText(QFileDialog.getExistingDirectory(self))
    
    
    def runDuplicateFinder(self):
        """\brief This method runs the duplicate finder and can be used as a 
                  SLOT.
        """
        
        # setup the search thread
        directory   = str(self.ui.lineEdit.text())
        self.search = SearchThread(directory)
        self.connect(self.search.df, QtCore.SIGNAL("progress"),
                                     self.setProgressValue)
        self.connect(self.search, QtCore.SIGNAL("finished()"),
                                  self.finishedSearch)        

        # start the search thread
        self.ui.statusbar.showMessage("Searching ...")
        self.search.start()


    def setProgressValue(self, value):
        """\brief This method sets the value for the progress bar."""
        self.ui.progressBar.setValue(value)


    def finishedSearch(self):
        """\brief This method is connected to the search thread and will be 
                  called once the search is finished.
        """
        self.ui.statusbar.showMessage("Search finished")
        self.ui.progressBar.setValue(0)

        # get the duplicate from last search thread
        duplicates = self.search.df.getDuplicates()

        # get the table widget and set the row count        
        table      = self.ui.tableWidget
        table.setRowCount(self.getRowCount())

        # set some initial parameter for the table loops
        row        = -1
        directory  = str(self.ui.lineEdit.text())
        
        # iterate over hash keys
        for key in duplicates:
            row += 1

            hashItem = QTableWidgetItem(key)
            table.setItem(row, 0, hashItem)

            # iterate over the file names of the current hash key
            for filename in duplicates[key]:
                row += 1
                
                filename  = filename.replace(directory, '.')
                valueItem = QTableWidgetItem(filename)
                table.setItem(row, 1, valueItem)


    def getRowCount(self):
        """\brief This method returns the number of rows needed to fit in the 
                  results of the duplicate files dictionary.
        """

        # get the duplicate from last search thread
        duplicates = self.search.df.getDuplicates()
        rowCount   = 0
        
        # iterate over hash keys
        for key in duplicates:
            rowCount += 1
            rowCount += len(duplicates[key])
        
        return rowCount