from PyQt4              import QtGui, QtCore
from DuplicateFinder    import DuplicateFinder


class DuplicateFinderUI(QtGui.QMainWindow):
    """\brief The DuplicateFinderUI class provides the main window for the 
              duplicate finder program.
    """
    
    
    def __init__(self):
        """\brief Constructor of the UI."""

        # constructor call of the parent class        
        QtGui.QMainWindow.__init__(self)
    
        # window settings    
        self.resize(400, 300)
        self.setWindowTitle('Duplicate Finder')

        # duplicate finder       
        self.currentPath = "C:\\Users\\ik3\\Documents"
        
        # create the ui
        self.createToolbar()
        self.createWidgets()


    def createToolbar(self):
        """\brief This method creates the initial toolbar of the ui."""

        # run button
        self.run = QtGui.QAction('Run', self)
        self.run.setShortcut('Ctrl+R')
        self.connect( self.run, QtCore.SIGNAL('triggered()'), 
                                self.runDuplicateFinder )

        # exit button
        self.exit = QtGui.QAction('Exit', self)
        self.exit.setShortcut('Ctrl+Q')
        self.connect( self.exit, QtCore.SIGNAL('triggered()'), 
                                 QtCore.SLOT('close()') )
    
        # toolbar
        self.toolbar = self.addToolBar('Toolbar')
        self.toolbar.addAction(self.run)
        self.toolbar.addAction(self.exit)
        
        
    def createWidgets(self):
        """\brief This method creates the initial widgets of the ui."""
        
        # edit field for current path
        labelEdit = QtGui.QLabel('Current Path')
        pathEdit  = QtGui.QLineEdit()
        self.connect(pathEdit, QtCore.SIGNAL('textChanged(QString)'), 
                               self.setCurrentPath)
        pathEdit.setText(self.currentPath)
        
        # result field
        self.results = QtGui.QTextEdit('')
        
        # horizontal container
        hbox       = QtGui.QHBoxLayout()
        hbox.addWidget(labelEdit)
        hbox.addWidget(pathEdit)       

        # vertical container
        vbox       = QtGui.QVBoxLayout()
        vbox.addLayout(hbox)
        vbox.addWidget(self.results)

        # container widget
        centralWidget = QtGui.QWidget(self)        
        centralWidget.setLayout(vbox)        

        # set container widget as central widget
        self.setCentralWidget(centralWidget)
        
    
    def runDuplicateFinder(self):
        """\brief This method runs the duplicate finder and can be used as a 
                  SLOT.
        """
        
        # create and run the duplicate finder
        df = DuplicateFinder(self.currentPath)
        df.run()
        duplicates = df.getDuplicates()

        # clear the result edit field from previous runs
        self.results.clear()
        
        # create a nice print out
        for key in duplicates:
            self.results.append(key)
            files = duplicates[key]
            for filename in files:
                self.results.append("\t%s" % filename)
            self.results.append("\n")

        
    def setCurrentPath(self, path):
        """\brief This method sets a new path and can be used as a SLOT."""
        self.currentPath = str(path)