# -*- coding: utf-8 -*-


"""
Kommunikation mit dem mobilen Roboter via serieller Schnittstelle
(emmuliert über USB)


"""

from numpy import clip
from ipHelp import IPS, ip_syshook

import serial
import time


"""
Grundidee: man sendet Befehle fester Länge (2-Byte):
erstes Byte: Befehl
zweites Byte: Daten
"""

class C(object):
    u = '0815'
    def p(self):
        print self.u


class RobotCommunicator(object):
    
    commands = {'forward': 'F',
                'backward': 'B',
                'sound':'S',
                'init': 'I'}
        
    def __init__(self, port = 0, baudrate = 9600, timeout=1):
        
        self.ser = serial.Serial(port, baudrate, timeout=timeout)

        self._test_communication() # Konsistenztest


    def _test_communication(self, output = True):
        rand_value = int((time.time()*100) %255)
        rand_byte = self._number_to_byte(rand_value)
        response = self._send(self.commands['init']+rand_byte)

        if output == False:
            return

        if response == rand_byte:
            print '-> Communication established successfully'
        else: 
            print 'Invalid initialization of serial communication'
            self.ser.flushInput()
            # assert False # abbruch erzwingen
        

    def _send(self, cmd, nores = False):

        assert len(cmd) == 2
        self.ser.write(cmd+'\n')

        if nores == True:
            response = ''
        else:
            response = self.ser.readline()[:-1] # cancel the trailing '\n'

        print 'cmd:     ', repr(cmd)
        print 'response:', repr(response)
        
        return response

    def _number_to_byte(self, x, min = 0, max = 255):
        x2 = clip(int(x), min, max)
        if x!=x2:
            print 'Invalid value: %s! Instead using: %s' %(str(x), str(x2))

        print ':::',x, x2, chr(x2)
        return chr(x2)


    def forward(self, x):
        databyte = self._number_to_byte(x)
        response = self._send(self.commands['forward']+databyte)
        

    def backward(self, x):
        databyte = self._number_to_byte(x)
        response = self._send(self.commands['backward']+databyte)

    def sound(self):
        # sound Kommando hat keine Daten
        self._send(self.commands['sound']+' ', nores=True)
        time.sleep(1.2)
        self.ser.flushInput()
        self._test_communication(output=False)
        

    def close(self):
        self.ser.close()

    def __del__(self):
        # Port schließen wenn Objekt gelöscht wird
        self.close()



RC = RobotCommunicator('/dev/ttyUSB0', timeout=2)       

#RC._init_communication()

IPS()